/*
 * fft2pc.c -- Character set translator for Final Fantasy Tactics by
 *	Squaresoft.  This is not intended for copyright infringement.
 *
 * Copyright 2000, George Greer <greerga@muc.muohio.edu>
 *
 * Licensed under the GNU General Public License (GPL)
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *   or visit ftp://ftp.gnu.org/pub/gnu/COPYING via the Internet.
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "mmaplib.h"

static const char *copyright = "fft2pc.c: Copyright 2000, George Greer <greerga@muc.muohio.edu>";
static const char *version = "v0.3 (Build: "__DATE__" "__TIME__")";

int fft2pc(const unsigned char *input, int length)
{
  int rpos;

  for (rpos = 0; rpos < length; rpos++) {
    if (input[rpos] <= 0x9)
      putchar('0' + input[rpos]);
    else if (input[rpos] <= 0x23)
      putchar('A' + (input[rpos] - 0xA));
    else if (input[rpos] <= 0x3D)
      putchar('a' + (input[rpos] - 0x24));
    else {
#if 1
      switch(input[rpos]) {
      case 0xD1:
        rpos++;
        switch(input[rpos]) {
        case 0x10: putchar('@'); break;		/* Really the "Reflect" arrow. */
        case 0x18: putchar('['); break;
        case 0x1A: putchar('!'); break;
        /* case 0x1B: "We left Goug Machine City[D1/1B]" */
        case 0x1D: putchar('-'); break;
        case 0x1E: putchar('+'); break;
        case 0x1F: fputs("<X>", stdout); break;	/* Cross Button */
        default:
          printf("[%X/%X]", input[rpos - 1], input[rpos]);
          break;
        }
        break;

      case 0xD9:
        rpos++;
        switch(input[rpos]) {
        case 0xB6: putchar('.'); break;
        case 0xB8: putchar('%'); break;
        case 0xB9: fputs("<O>", stdout); break;	/* Circle Button */
        case 0xBC: putchar('.'); break;		/* Really a . in center of line vertically. */
        case 0xBD: putchar(':'); break;
        case 0xBE: putchar('('); break;
        case 0xBF: putchar(')'); break;
        case 0xC0: putchar('"'); break;
        case 0xC1: putchar('\''); break;
        case 0xC4: putchar(']'); break;
        case 0xC6: putchar('/'); break;
        case 0xC7: fputs("<T>", stdout); break;	/* Triangle Button */
        case 0xC8: fputs("<S>", stdout); break; /* Square Button */
        case 0xC9: putchar('?'); break;
        default:
          printf("[%X/%X]", input[rpos - 1], input[rpos]);
          break;
        }
        break;

      case 0xDA:
        rpos++;
        switch(input[rpos]) {
        case 0x73: putchar(' '); break;	/* Really?? */
        case 0x74: putchar(','); break;
        case 0x75: putchar(':'); break;
        case 0x76: putchar('\''); break;	/* Three ' now. Sheesh. */
        case 0x77: putchar('\"'); break;	/* Another one? */
        default:
          printf("[%X/%X]", input[rpos - 1], input[rpos]);
          break;
        }
        break;

      case 0xE3:
        rpos++;
        switch(input[rpos]) {
        case 0: fputs("<Normal Text>", stdout); break;
        case 4: fputs("<Gray Text>", stdout); break;
        case 8: fputs("<Red Text>", stdout); break;
        default:
          printf("[%X/%X]", input[rpos - 1], input[rpos]);
          break;
        }
        break;

      case 0x3E: putchar('!'); break;		/* Another !, why? */
      case 0x40: fputs("!?", stdout); break;	/* "God! Why are you testing me[40]" */
      case 0x42: putchar('+'); break;		/* Another + */
      case 0x91: putchar('"'); break;		/* Yet another one. */
      case 0x93: putchar('\''); break;		/* One ' isn't enough? (See job results.) */
      case 0xE0: fputs("<Ramza>", stdout); break;
#if 0
      case 0xE4: fputs("<Variable: Age??>", stdout); break;
      case 0xEB: fputs("<Variable: Month??/JobPerson??>", stdout); break;
#endif
      case 0xEC: printf("<Picture #%u>", input[++rpos]); break;
      case 0xF8: putchar('\n'); break;		/* In-text */
      case 0xFA: putchar(' '); break;
      case 0xFE: putchar('\n'); putchar('\n'); break;	/* \0 */

      default:
        printf("[%X]", input[rpos]);
        break;
      }
#else
      printf("[%X]", input[rpos]);
#endif
    } /* if */
  } /* for */
  return rpos;
}

int main(int argc, char *argv[])
{
  void *handle;
  char *filename;

  if (argc < 2) {
    printf("%s filename\n\tor\n%s -V (for version and copyright)\n", argv[0], argv[0]);
    return 1;
  }

  if (!strcmp(argv[1], "-V")) {
    printf("%s\n%s: version %s\n", copyright, argv[0], version);
    return 1;
  }

  filename = argv[1];

  if (!(handle = mapopen(filename)))
    error(filename);
  fft2pc(mapdata(handle), maplength(handle));
  mapclose(handle);

  return 0;
}
